local HealingTracker = _G.HealingTracker
local Combat = HealingTracker.Combat
local Segments = HealingTracker.Segments

function Combat:Start()

	HealingTracker.Combat.CombatSeconds = 0
	HealingTracker.ProfileOptions.Recording = 1
	Combat.Summons = {}

	local segment = Segments:Begin(nil)

	if HealingTracker.ProfileOptions.WindowShown and HealingTracker.ProfileOptions.HideInCombat == 1 then
		HealingTracker.TrackerFrame:Hide()
	end
	
	HealingTracker.TrackerFrame:SetSegment(segment)
	HealingTracker.TrackerFrame:UpdateBars()
	
end

function Combat:End()

	HealingTracker.ProfileOptions.Recording = 0
	Combat.Summons = {}
	
	if HealingTracker.ProfileOptions.WindowShown and HealingTracker.ProfileOptions.HideInCombat == 1 then
		HealingTracker.TrackerFrame:Show()
	end
	
	local willbeSaved = 0
	if HealingTracker.ProfileOptions.Keep > 0 and Combat.CombatSeconds > HealingTracker.ProfileOptions.KeepDuration then
		if Segments.Combat.IsBoss == 1 or HealingTracker.ProfileOptions.Keep == 2 then
			willbeSaved = 1
		end
	end
	
	if willbeSaved == 1 then
		local name, itype, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceMapId = GetInstanceInfo()
		if itype == "pvp" or itype == "arena" then
			willbeSaved = 0
		end
	end

	Segments:FinishCurrent(willbeSaved)
end

function Combat:OnSpellHeal(spellId, caster, target, casterGuid, targetGuid, spellName, amount, overhealing, absorbed, scritical, periodic)

	local pdb = HealingTracker.ProfileOptions

    --calculating hp before heal
	local hp = UnitHealth(target)
	local heal = amount - overhealing

	if overhealing > 0 then
		Segments:AddOverHeal(caster, target, casterGuid, spellId, spellName, overhealing, scritical)
	end

	local firstHp = hp - heal
	local maxHp = UnitHealthMax(target)
	local ratio = firstHp / maxHp
	
	if heal < 1 then return end

	--if the healind is not important for saving someone from death, return
	if ratio > pdb.PointlessLimit then
		Segments:AddPointlessHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, periodic)
		HealingTracker.TrackerFrame:UpdateBars()
		return
	end
	
	local diffRatio = pdb.PointlessLimit - ratio
	local hpPercent = heal / maxHp
	
	--if pdb.IgnoreMinimals and hpPercent < 0.03 then
	if hpPercent < 0.02 then
		Segments:AddPointlessHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, periodic)
		HealingTracker.TrackerFrame:UpdateBars()
		return
	end
	if hpPercent > 0.60 then
		hpPercent = 0.60
	end
	
	local multiplyCalculation = 0
	local sumCalculation = 0
	local calculated = 0
	
	if pdb.Calculation == nil then pdb.Calculation = 1 end

	if pdb.Calculation == 1 then
		local mx = math.floor(diffRatio * diffRatio * hpPercent * 300)
		local sx = math.floor((diffRatio + hpPercent) * 10)
		if mx > 50 then mx = 50 end
		if sx > 50 then sx = 50 end
		
		if mx > 0 then multiplyCalculation = HealingTracker.Constants.EmergencyTable[mx] end
		if sx > 0 then sumCalculation = HealingTracker.Constants.RegularTable[sx] end
		
		if periodic then
			multiplyCalculation = math.floor(multiplyCalculation * 0.82)
			sumCalculation = math.floor(sumCalculation * 0.66)
		end

		calculated = multiplyCalculation + sumCalculation
		
	elseif pdb.Calculation == 2 then
		multiplyCalculation = math.floor(diffRatio * hpPercent * 100.00 * 1.68)
		sumCalculation = math.floor((diffRatio + hpPercent) * 10.00 * 1.2)
		calculated = multiplyCalculation + sumCalculation
	
	end
	
	if calculated < 1 then
		Segments:AddPointlessHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, periodic)
		HealingTracker.TrackerFrame:UpdateBars()
		return
	end
	
	-- 1.5 scale constant
	local point = calculated
	local percInt = math.floor(ratio * 100)
	local message = ""
	Segments:AddAidfulHeal(caster, target, casterGuid, spellId, spellName, amount, heal, scritical, point, percInt, multiplyCalculation, sumCalculation, periodic)
	HealingTracker.TrackerFrame:UpdateBars()
	
end
