﻿local HealingTracker = _G.HealingTracker

local LSM = LibStub("LibSharedMedia-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local AceGUI = LibStub("AceGUI-3.0")
local boss = LibStub("LibBossIDs-1.0")
--local L = LibStub("AceLocale-3.0"):GetLocale("HealingTracker")

local RAID_FLAGS = COMBATLOG_OBJECT_TYPE_PLAYER + COMBATLOG_OBJECT_AFFILIATION_MINE + COMBATLOG_OBJECT_AFFILIATION_PARTY + COMBATLOG_OBJECT_AFFILIATION_RAID	--mobile flags

function HealingTracker:OnInitialize()

	self.db = LibStub("AceDB-3.0"):New("HealingTrackerDB", _G.DefaultOptions)

	if self.db.profile == nil or not self.db.profile.Init then
		HealingTracker:ApplyDefaults()
	end

	if HealingTracker.ProfileOptions == nil then
		HealingTracker.ProfileOptions = self.db.profile
	end
	
	HealingTracker.ProfileOptions.Recording = 0
	HealingTracker.Combat.CombatSeconds = 0
	
	if not HealingTracker.ProfileOptions.WindowShown then
		HealingTracker.ProfileOptions.WindowShown = 1
	end
		
	LSM:Register("statusbar", "Aluminium", "Interface\\AddOns\\HealingTracker\\textures\\Aluminium")
	LSM:Register("statusbar", "Armory", "Interface\\Addons\\HealingTracker\\textures\\Armory")
	LSM:Register("statusbar", "BantoBar", "Interface\\Addons\\HealingTracker\\textures\\BantoBar")
	LSM:Register("statusbar", "Flat", "Interface\\Addons\\HealingTracker\\textures\\Flat")
	LSM:Register("statusbar", "Gloss", "Interface\\Addons\\HealingTracker\\textures\\Gloss")
	LSM:Register("statusbar", "Graphite", "Interface\\Addons\\HealingTracker\\textures\\Graphite")
	LSM:Register("statusbar", "Minimalist", "Interface\\Addons\\HealingTracker\\textures\\Minimalist")
	LSM:Register("statusbar", "Otravi", "Interface\\Addons\\HealingTracker\\textures\\Otravi")
	LSM:Register("statusbar", "Perl", "Interface\\Addons\\HealingTracker\\textures\\Perl")
	LSM:Register("statusbar", "Smooth", "Interface\\Addons\\HealingTracker\\textures\\Smooth")
	
	HealingTracker.TrackerFrame:Init()
	HealingTracker.OptionsFrame:Init()
	HealingTracker.DetailsFrame:Init()
	HealingTracker.ReportFrame:Init()

end


function HealingTracker:ApplyDefaults(partial)
	
	self.db.profile.Init = 1
	self.db.profile.Active = 1
	self.db.profile.Keep = 1
	self.db.profile.Recording = 0
	self.db.profile.HideInCombat = 0
	self.db.profile.WindowShown = 1
	self.db.profile.DisableHidden = 0
	self.db.profile.PointlessLimit = 0.60
	self.db.profile.KeepDuration = 0
	self.db.profile.IgnoreMinimals = 0
	self.db.profile.CurrentSegment = 0
	self.db.profile.Calculation = 1
	self.db.profile.MergeTrash = true
	if not partial then
		self.db.profile.Bounds = {
			Point = "CENTER",
			RelativePoint = "CENTER",
			XOff = 0,
			YOff = 0,
			Width = 180,
			Height = 200
		}
	end
	self.db.profile.Bars = {
		Texture = "Minimalist",
		Height = 19,
		Font = "Arial Narrow",
		FontSize = 10,
		FontColor = {R=1 ,G=1, B=1, A=1}
	}
	self.db.profile.Columns = {
		Right = 1,
		Mid = 3,
		Left = 0
	}
	self.db.profile.Segments = {}
	self.db.profile.HeaderButtons = {
		Reports = true,
		Options = true,
		Reset = true,
		Close = true
	}
	HealingTracker.ProfileOptions = self.db.profile
	HealingTracker.TrackerFrame:Redraw()
	HealingTracker.TrackerFrame:UpdateHeights()
	HealingTracker.TrackerFrame:UpdateBars()

end

function HealingTracker:OnEnable()
	
	if HealingTracker.ProfileOptions == nil then
		HealingTracker:ApplyDefaults()
	end

	HealingTracker.ProfileOptions.Recording = 0
	HealingTracker.Combat.CombatSeconds = 0

	DEFAULT_CHAT_FRAME:AddMessage("Healing Tracker, Néd <Warpath> @ EU-Twisting Nether, Commands: /ht or /htr", 0, 0.6, 1)
	
	self:RegisterChatCommand("ht", "Command")
	self:RegisterChatCommand("htr", "GermanCommand")
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ENCOUNTER_START")
	self:RegisterEvent("ENCOUNTER_END")
	
	if self.db.profile.WindowShown then
		HealingTracker.TrackerFrame:Show()
	else
		HealingTracker.TrackerFrame:Hide()
	end
	
end

function HealingTracker:OnDisable()

	HealingTracker.TrackerFrame:Hide()
	HealingTracker.OptionsFrame:Hide()
	HealingTracker.DetailsFrame:Hide()
	HealingTracker.ReportFrame:Hide()

	self:CancelTimer(HealingTracker.Combat.Timer)

end

function HealingTracker:Command(param)
	HealingTracker:FullCommand(param)
end

function HealingTracker:GermanCommand(param)
	HealingTracker:FullCommand(param)
end

function HealingTracker:FullCommand(param)
	if param == "show" then HealingTracker.TrackerFrame:Show()
	elseif param == "hide" then HealingTracker.TrackerFrame:Hide()
	elseif param == "reset" then HealingTracker:Reset(0)
	elseif param == "options" then HealingTracker.OptionsFrame:Show()
	elseif param == "default" then HealingTracker:ApplyDefaults()
	else
		self:Print("Usage: /ht or /htr")
		self:Print(("%-20s"):format("/ht show"))
		self:Print(("%-20s"):format("/ht hide"))
		self:Print(("%-20s"):format("/ht reset"))
		self:Print(("%-20s"):format("/ht options"))
		self:Print(("%-20s"):format("/ht default"))
		
	end
end

function HealingTracker:Reset(rtype)
	if rtype == 1 then
		HealingTracker.Segments:ClearTrash()
	elseif rtype == 2 then
		HealingTracker.Segments:ClearCurrent()
	else
		HealingTracker.Segments:Clear()
	end
	HealingTracker.TrackerFrame:Redraw()
	HealingTracker.TrackerFrame:UpdateHeights()
	HealingTracker.TrackerFrame:UpdateBars()
end

function HealingTracker:TimerEvent()
	HealingTracker.Segments:UpdateCombatHealers(HealingTracker.Combat.CombatSeconds)
	HealingTracker.Combat.CombatSeconds = HealingTracker.Combat.CombatSeconds + 1
end

function HealingTracker:ENCOUNTER_START(event, encounterID, name, difficulty, size)

	if HealingTracker.ProfileOptions.Recording == 1 then
		HealingTracker.Combat:End()
		HealingTracker:CancelTimer(HealingTracker.Combat.Timer)
	end
	HealingTracker.Combat:Start()
	HealingTracker.Combat.Timer = self:ScheduleRepeatingTimer("TimerEvent", 1)
	HealingTracker.Segments.Combat.IsBoss = 1
	HealingTracker.Segments.Combat.EncounterId = encounterID
	if HealingTracker.CurrentInstance.Data then
		HealingTracker.Segments.Combat.Title = HealingTracker.CurrentInstance.Data.Abbreviation..": "..name
	else
		HealingTracker.Segments.Combat.Title = name
	end
end

function HealingTracker:ENCOUNTER_END(event, encounterID, name, difficulty, size, success)
	if HealingTracker.Segments.Combat then
		local wp = 1
		if success == 1 then
			wp = 0
		end
		HealingTracker.Segments.Combat.IsWipe = wp
		HealingTracker.Combat:End()
		HealingTracker:CancelTimer(HealingTracker.Combat.Timer)
	end
end

function HealingTracker:PLAYER_ENTERING_WORLD()
	local name, itype, difficulty, difficultyName, maxPlayers, playerDifficulty, isDynamicInstance, instanceMapId, instanceGroupSize = GetInstanceInfo()
	HealingTracker.CurrentInstance = {
		Id = instanceMapId,
		Name = name,
		Type = itype,
		DiffIndex = difficultyIndex,
		DiffName = difficultyName,
		Data = nil
	}
	local idata = HealingTracker.Constants.Instances[instanceMapId]
	if idata then
		HealingTracker.CurrentInstance.Data = idata
	end
end

function HealingTracker:PLAYER_REGEN_DISABLED()
	if HealingTracker.ProfileOptions.Recording == 1 then
		return
	end
	local iname, itype = GetInstanceInfo()
	if itype == "pvp" or itype == "arena" then
		return
	end
	HealingTracker.Combat:Start()
	HealingTracker.Combat.Timer = self:ScheduleRepeatingTimer("TimerEvent", 1)
	if HealingTracker.CurrentInstance.Data then
		HealingTracker.Segments.Combat.Title = HealingTracker.CurrentInstance.Data.Abbreviation..": Trash"
	else
		HealingTracker.Segments.Combat.Title = "Trash"
	end
	HealingTracker.Segments.Combat.IsBoss = 0
end

function HealingTracker:PLAYER_REGEN_ENABLED()
	if HealingTracker.Segments.Combat then
		if HealingTracker.Segments.Combat.IsBoss == 0 then
			HealingTracker.Combat:End()
			HealingTracker:CancelTimer(HealingTracker.Combat.Timer)
		end
	end
end

function HealingTracker:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventtype, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, ...)

	if HealingTracker.ProfileOptions.Recording ~= 1 then return end
	if HealingTracker.ProfileOptions.WindowShown == false then return end

	local srcFilter = bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) + bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) + bit.band(srcFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY)
	local dstFilter = bit.band(dstFlags, COMBATLOG_OBJECT_AFFILIATION_MINE) + bit.band(dstFlags, COMBATLOG_OBJECT_AFFILIATION_RAID) + bit.band(dstFlags, COMBATLOG_OBJECT_AFFILIATION_PARTY)
	
	if srcFilter < 1 and dstFilter < 1 then return end
	if eventtype == "SPELL_DAMAGE" then return end

	if eventtype ~= "SPELL_SUMMON" and eventtype ~= "SPELL_ABSORBED" and eventtype ~= "SPELL_HEAL" and eventtype ~= "SPELL_PERIODIC_HEAL" and eventtype ~= "SPELL_CAST_SUCCESS" then return end
	
	local src = bit.band(srcFlags,RAID_FLAGS)
	local dst = bit.band(dstFlags,RAID_FLAGS)
	local psrc = bit.band(srcFlags, COMBATLOG_OBJECT_TYPE_PET + COMBATLOG_OBJECT_TYPE_GUARDIAN)
	local pdst = bit.band(dstFlags, COMBATLOG_OBJECT_TYPE_PET + COMBATLOG_OBJECT_TYPE_GUARDIAN)
	
	if psrc ~= 0 then src = 0 end
	if pdst ~= 0 then dst = 0 end
	if src == 0 and dst == 0 then return end

	if eventtype == "SPELL_CAST_SUCCESS" and UnitIsPlayer(srcName) then
		local spellId, spellName = ...
		if spellId == HealingTracker.Constants.Innervate then
			HealingTracker.Segments:AddCooldown(spellId, dstName, srcName, dstGUID, srcGUID, spellName, 1)
		else
			for _,v in pairs(HealingTracker.Constants.TargetSpells) do
				if v == spellId then
					HealingTracker.Segments:AddCooldown(spellId, srcName, dstName, srcGUID, dstGUID, spellName, 1)
					break
				end
			end
			for _,v in pairs(HealingTracker.Constants.NonTargetSpells) do
				if v == spellId then
					HealingTracker.Segments:AddCooldown(spellId, srcName, dstName, srcGUID, dstGUID, spellName, 0)
					break
				end
			end
		end
	end
	
	local isPeriodic = 0
	local isAbsorbed = 0
	if eventtype == "SPELL_PERIODIC_HEAL" then
		isPeriodic = 1
	end
	
	if eventtype == "SPELL_SUMMON" then
		local spellId, spellName = ...
		if spellId == 198838 or spellId == 198664 then
			HealingTracker.Combat.Summons[#HealingTracker.Combat.Summons + 1] = {
				PlayerGuid = srcGUID,
				PlayerName = srcName,
				SummonId = dstGUID,
				SummonName = dstName
			}
		end
	end

	-- spell absorb
	if eventtype == "SPELL_ABSORBED" then
		isAbsorbed = 1
		local aplayerGuid, aplayerName, aspellId, aspellName, absorbAmount
		local a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13 = ...
		if type(a1) == "number" then
			--self:Print("a1:"..a1.." a2:"..a2.." a3:"..a3.." a4:"..a4.." a5:"..a5.." a6:"..a6.." a7:"..a7.." a8:"..a8.." a9:"..a9.." a10:"..a10.." a11:"..a11)
			aplayerGuid = a4
			aplayerName = a5
			aspellId = a8
			aspellName = a9
			absorbAmount = a11
		else
			aplayerGuid = a1
			aplayerName = a2
			aspellId = a5
			aspellName = a6
			absorbAmount = a8
			--self:Print("a1:"..a1.." a2:"..a2.." a3:"..a3.." a4:"..a4.." a5:"..a5.." a6:"..a6.." a7:"..a7.." a8:"..a8)
		end
		if UnitIsPlayer(dstName) then
			if UnitIsPlayer(aplayerName) then
				HealingTracker.Combat:OnSpellHeal(aspellId, aplayerName, dstName, aplayerGuid, dstGUID, aspellName, absorbAmount, 0, 1, nil, nil)	
			else
				for _,v in pairs(HealingTracker.Combat.Summons) do
					if v.SummonId == aplayerGuid then
						HealingTracker.Combat:OnSpellHeal(aspellId, v.PlayerName, dstName, v.PlayerGuid, dstGUID, aspellName, absorbAmount, 0, 1, nil, nil)	
						break
					end
				end
			end
		end

	-- spell heal
	elseif (isPeriodic == 1 or eventtype == "SPELL_HEAL") and dst ~= 0 and src ~= 0 then
		if UnitIsPlayer(srcName) and UnitIsPlayer(dstName) then
			local spellId, spellName, spellSchool, samount, soverhealing, absorbed, scritical = ...
			HealingTracker.Combat:OnSpellHeal(spellId, srcName, dstName, srcGUID, dstGUID, spellName, samount, soverhealing, 0, scritical, isPeriodic)
		end
		return
	end
	
end
